use std::sync::Arc;

use app_state::AppState;
use calibre_db::calibre::Calibre;
use config::Config;
use poem::{
    endpoint::EmbeddedFilesEndpoint, get, listener::TcpListener, middleware::Tracing, EndpointExt,
    Route, Server,
};
use rust_embed::RustEmbed;

pub mod app_state;
pub mod basic_auth;
pub mod cli;
pub mod config;
pub mod data {
    pub mod book;
}
pub mod handlers {
    pub mod author;
    pub mod authors;
    pub mod books;
    pub mod cover;
    pub mod download;
    pub mod error;
    pub mod opds;
    pub mod paginated;
    pub mod recents;
    pub mod series;
    pub mod series_single;
}
pub mod opds;
pub mod templates;

#[derive(RustEmbed)]
#[folder = "static"]
pub struct Files;

pub async fn run(config: Config) -> Result<(), std::io::Error> {
    let calibre = Calibre::load(&config.metadata_path).expect("failed to load calibre database");
    let app_state = Arc::new(AppState { calibre, config });

    let app = Route::new()
        .at("/", get(handlers::recents::handler))
        .at("/opds", get(handlers::opds::handler))
        .at("/books", get(handlers::books::handler_init))
        .at("/books/:cursor/:sort_order", get(handlers::books::handler))
        .at("/series", get(handlers::series::handler_init))
        .at(
            "/series/:cursor/:sort_order",
            get(handlers::series::handler),
        )
        .at("/series/:id", get(handlers::series_single::handler))
        .at("/authors", get(handlers::authors::handler_init))
        .at("/authors/:id", get(handlers::author::handler))
        .at(
            "/authors/:cursor/:sort_order",
            get(handlers::authors::handler),
        )
        .at("/cover/:id", get(handlers::cover::handler))
        .at("/book/:id/:format", get(handlers::download::handler))
        .nest("/static", EmbeddedFilesEndpoint::<Files>::new())
        .data(app_state)
        .with(Tracing);

    Server::new(TcpListener::bind("[::]:3000"))
        .name("cops-web")
        .run(app)
        .await
}
