use std::io::Cursor;

use quick_xml::{
    events::{BytesDecl, BytesStart, Event},
    se::to_string,
    Reader, Writer,
};
use serde::Serialize;
use time::OffsetDateTime;

use super::{author::Author, entry::Entry, error::OpdsError, link::Link};

#[derive(Debug, Serialize)]
#[serde(rename = "feed")]
pub struct Feed {
    pub title: String,
    pub id: String,
    #[serde(with = "time::serde::rfc3339")]
    pub updated: OffsetDateTime,
    pub icon: String,
    pub author: Author,
    #[serde(rename = "link")]
    pub links: Vec<Link>,
    #[serde(rename = "entry")]
    pub entries: Vec<Entry>,
}

impl Feed {
    pub fn as_xml(&self) -> Result<String, OpdsError> {
        let xml = to_string(&self)?;
        let mut reader = Reader::from_str(&xml);
        reader.trim_text(true);

        let declaration = BytesDecl::new("1.0", Some("UTF-8"), None);
        let mut writer = Writer::new(Cursor::new(Vec::new()));
        writer.write_event(Event::Decl(declaration))?;

        let mut feed_start = BytesStart::new("feed");
        feed_start.push_attribute(("xmlns", "http://www.w3.org/2005/Atom"));
        feed_start.push_attribute(("xmlns:xhtml", "http://www.w3.org/1999/xhtml"));
        feed_start.push_attribute(("xmlns:opds", "http://opds-spec.org/2010/catalog"));
        feed_start.push_attribute(("xmlns:opensearch", "http://a9.com/-/spec/opensearch/1.1/"));
        feed_start.push_attribute(("xmlns:dcterms", "http://purl.org/dc/terms/"));
        feed_start.push_attribute(("xmlns:thr", "http://purl.org/syndication/thread/1.0"));

        loop {
            match reader.read_event() {
                Ok(Event::Start(e)) if e.name().as_ref() == b"feed" => {
                    writer.write_event(Event::Start(feed_start.clone()))?
                }
                Ok(Event::Eof) => break,
                Ok(e) => writer.write_event(e)?,
                Err(e) => return Err(e)?,
            }
        }
        let result = writer.into_inner().into_inner();
        Ok(String::from_utf8(result)?)
    }
}
